/********************************************************************
 *																	*
 *		Data Dictionary management class							*
 *																	*
 ********************************************************************/

#ifndef __DICT_H
#define __DICT_H

_CLASSDEF(TddFile)
_CLASSDEF(TddField)
_CLASSDEF(TddIndex)
_CLASSDEF(Tdd)

//*******************************************************************
//	Dictionary mask bits
const short	Xf_Dictionary = 0x0010;			// set if file is a dictionary
const short	Xe_CaseSens = 0x0001;			// set if case insensitive string type

enum openMode {
	boExclusive = -4,						// exlusive access to the database
	boVerify = -3,							// open using dos verify
	boReadOnly = -2,						// open table in read-only mode
	boAccelerated = -1,						// open in accelerated mode
	boNormal = 0 };							// open in normal mode

enum createMode {
	bcCreate = 0,							// unconditional create
	bcNoOverwrite = -1,						// don't create if existing
	bcCreateNonTransactional = -2,			// create unconditionally non-transactional (server only)
	bcCreateNoOverwtiteTrans = -3 };		// don't create non-transacational if existing

struct verBlock {
	short	version;						// version number
	short	revision;						// revisiion number
	char	type;							// N=server
											// P=Btrieve OS/2
											// W=Btrieve windows
};

const int btReadOnly = 0x0001;				// read-only mode
const int btEncrypted = 0x0002;				// encrypted file

extern const char altSeq[];

const int Xc_VarLength = 0x0001;			// variable length fields allowed
const int Xc_PreAllocate = 0x0004;			// pre-allocate file

class fileSpec {							// used to create a file spec
public:
	inline fileSpec()						// constructor
		{ memset(this, 0, sizeof( fileSpec )); }
	inline fileSpec(short r, short p, short c, short f, short a=0)
		{
		memset(this, 0, sizeof( fileSpec ));
		recordLength = r;
		pageSize = p;
		indexCount = c;
		flags = f;
		allocationSize = a;
		}

	short	recordLength;					// size of a record
	short	pageSize;						// size of a page
	short	indexCount;						// # indexes
	long	res1;							// reserved
	short	flags;							// record flags
	short	res2;							// reserved
	short	allocationSize;					// # pages to pre allocate
};

class keySpec {							// used to create a key spec
public:
	inline keySpec()
		{ memset(this, 0, sizeof( keySpec )); }
	inline keySpec(short p, short l, short f, char t=0, char n=0)
		{
		memset(this, 0, sizeof( keySpec ));
		position = p+1;
		length = l;
		flags = f;
		type = t;
		null=n;
		}

	short	position;						// use the fldOffset macro
	short	length;							// bytes in the key
	short	flags;							// key flags
	long	res1;
	char	type;							// extended type
	char	null;							// null value
	long	res2;
};

struct altCol {
	char	signature;						// signature byte
	char	name[8];						// sequence name
	char	table[256];						// collating sequence

	inline altCol()
		{
		memset(this, 0, sizeof( altCol ));
        signature = 0xAC;
		}
	inline altCol(const char *n, const char *t = 0)
		{
		signature = 0xAC;
		memset(name, 0, 8);					// init name
		strncpy(name, n, 8);				// copy new name (max 8 chars)
		memcpy(table, t ? t : altSeq, 256);	// copy the given sequence else use upper case tbl
		}
};

int addTable(Tdd& dict, const string &name, const string &path, short flags);
int addField(Tdd& dict, short fileId, const string &name, short type, short offset, short len, short dec=0);
int addInSeg(Tdd& dict, short fileId, short fieldId, short index, short part, short flags);
int addIndex(Tdd& dict, short fileId, short index, const string &name);

//*******************************************************************
//		Database Management class ---- loads and maintains the dictionary

class Tdd {									// this is the dictionary controls
public:
	Tdd();
	Tdd(const string &path);
	~Tdd();

	void setPath(const string &path);			// set data dictionary path
	int open();								// open data dictionary
	int create();							// create data dictionary
	int close();							// close dictionary files

	TddFile& getFile() const				// return file
		{ return *ddFile; }
	short getFileId() const;

	TddField& getField() const
		{ return *ddField; }
	short getFieldId() const;

	TddIndex& getIndex() const
		{ return *ddIndex; }

	// Dictionary functions.
    // tables...
	int addTable(const string &name, const string &path);	// add a table to the dictionary
	int dropTable(short Id) const;			// drop a table from the dictionary
	int getTableId(const string &name) const;	// return the dictionary Id for a table
	const char *getTablePath(short Id) const;	// get the path name
    // Fields...
	int addField(short file, const string &name, char type, short offset, short size, char dec=0);
	int dropField(short field) const;		// remove a field from the dictionary
	int dropAllFields(short file) const;	// remove all fields
	int getFieldId(short file, const string &name) const; 	// gets the field Id for a field
	int getFieldOffset(short Id) const;		// get the field offset
	int getFieldLength(short Id) const;		// get the length of the field
	int getFieldType(short Id) const;		// get the type of the field
	// Keys...
	int addKey(short file, short num, const string &name);
	int addSeg(short file, short field, short num, short seq, short flags);
	int dropKey(short indexId) const;		// remove a key from the dictionary
	int getKeyId(short file, const string &name) const;	// get the key Id
	int getKeyNum(short Id) const;			// get the key number in the file
	int getSeg(short file, short index, short seg) const;	// gets the index segment field number requested
	int getFlags(short file, short index, short seg) const;	// gets the flags for the segment requested

	const char * getUprCase() const;		// return an upper case collating table

	// set the function to call at the bottom of loops.  allows
	// windows to multi-task when the system is busy
	void setLoop(void *obj=0, void (*func)(void *, int, long)=0) const;

private:
	int			ddOpen;						// true when opened
	PTddFile 	ddFile;						// file control structures
	PTddField 	ddField;					// field controls
	PTddIndex	ddIndex;					// index controls
};

#endif